@@ -26,12 +26,8 @@
 
 #include <dbus/dbus-glib.h>
 
-gboolean _dbus_gmain_test   (const char *test_data_dir);
 gboolean _dbus_gobject_test (const char *test_data_dir);
-gboolean _dbus_gutils_test  (const char *test_data_dir);
-gboolean _dbus_glib_test    (const char *test_data_dir);
 gboolean _dbus_gvalue_test  (const char *test_data_dir);
-gboolean _dbus_gvalue_utils_test    (const char *test_data_dir);
 
 void dbus_glib_internal_do_not_use_run_tests (const char *test_data_dir);
 
