@@ -17,7 +17,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -845,6 +845,164 @@ array_constructor (GType type)
   return array;
 }
 
+static void
+array_iterator (GType garray_type,
+    gpointer instance,
+    DBusGTypeSpecializedCollectionIterator iterator,
+    gpointer user_data)
+{
+  GArray *array;
+  GType elt_gtype;
+  guint i;
+
+  array = instance;
+
+  elt_gtype = dbus_g_type_get_collection_specialization (garray_type);
+
+  for (i = 0; i < array->len; i++)
+    {
+      GValue val = {0, };
+      g_value_init (&val, elt_gtype);
+
+      switch (elt_gtype)
+        {
+          case G_TYPE_BOOLEAN:
+            g_value_set_boolean (&val, !!g_array_index (array, gboolean, i));
+            break;
+
+          case G_TYPE_FLOAT:
+            g_value_set_float (&val, g_array_index (array, gfloat, i));
+            break;
+
+          case G_TYPE_DOUBLE:
+            g_value_set_double (&val, g_array_index (array, gdouble, i));
+            break;
+
+          case G_TYPE_CHAR:
+            g_value_set_char (&val, g_array_index (array, gchar, i));
+            break;
+
+          case G_TYPE_UCHAR:
+            g_value_set_uchar (&val, g_array_index (array, guchar, i));
+            break;
+
+          case G_TYPE_INT:
+            g_value_set_int (&val, g_array_index (array, gint, i));
+            break;
+
+          case G_TYPE_UINT:
+            g_value_set_uint (&val, g_array_index (array, guint, i));
+            break;
+
+          case G_TYPE_LONG:
+            g_value_set_long (&val, g_array_index (array, glong, i));
+            break;
+
+          case G_TYPE_ULONG:
+            g_value_set_ulong (&val, g_array_index (array, gulong, i));
+            break;
+
+          case G_TYPE_INT64:
+            g_value_set_int64 (&val, g_array_index (array, gint64, i));
+            break;
+
+          case G_TYPE_UINT64:
+            g_value_set_uint64 (&val, g_array_index (array, guint64, i));
+            break;
+
+          default:
+            g_assert_not_reached ();
+        }
+
+      iterator (&val, user_data);
+    }
+}
+
+static void
+array_append (DBusGTypeSpecializedAppendContext *ctx,
+    GValue *value)
+{
+  GArray *array = g_value_get_boxed (ctx->val);
+  GType elt_gtype;
+  union {
+      guint64 u64;
+      gint64 i64;
+      gulong ul;
+      glong l;
+      guint u;
+      gint i;
+      guchar uc;
+      gchar c;
+      gboolean b;
+      gfloat f;
+      gdouble d;
+  } tmp;
+
+  elt_gtype = dbus_g_type_get_collection_specialization (
+      G_VALUE_TYPE (ctx->val));
+
+  switch (elt_gtype)
+    {
+      case G_TYPE_BOOLEAN:
+        tmp.b = g_value_get_boolean (value);
+        g_array_append_val (array, tmp.b);
+        break;
+
+      case G_TYPE_FLOAT:
+        tmp.f = g_value_get_float (value);
+        g_array_append_val (array, tmp.f);
+        break;
+
+      case G_TYPE_DOUBLE:
+        tmp.d = g_value_get_double (value);
+        g_array_append_val (array, tmp.d);
+        break;
+
+      case G_TYPE_CHAR:
+        tmp.c = g_value_get_char (value);
+        g_array_append_val (array, tmp.c);
+        break;
+
+      case G_TYPE_UCHAR:
+        tmp.uc = g_value_get_uchar (value);
+        g_array_append_val (array, tmp.uc);
+        break;
+
+      case G_TYPE_INT:
+        tmp.i = g_value_get_int (value);
+        g_array_append_val (array, tmp.i);
+        break;
+
+      case G_TYPE_UINT:
+        tmp.u = g_value_get_uint (value);
+        g_array_append_val (array, tmp.u);
+        break;
+
+      case G_TYPE_LONG:
+        tmp.l = g_value_get_long (value);
+        g_array_append_val (array, tmp.l);
+        break;
+
+      case G_TYPE_ULONG:
+        tmp.ul = g_value_get_ulong (value);
+        g_array_append_val (array, tmp.ul);
+        break;
+
+      case G_TYPE_INT64:
+        tmp.i64 = g_value_get_int64 (value);
+        g_array_append_val (array, tmp.i64);
+        break;
+
+      case G_TYPE_UINT64:
+        tmp.u64 = g_value_get_uint64 (value);
+        g_array_append_val (array, tmp.u64);
+        break;
+
+      default:
+        g_assert_not_reached ();
+    }
+}
+
 static gpointer
 array_copy (GType type, gpointer src)
 {
@@ -1141,8 +1299,8 @@ _dbus_g_type_specialized_builtins_init (void)
       NULL,
     },
     array_fixed_accessor,
-    NULL,
-    NULL,
+    array_iterator,
+    array_append,
     NULL
   };
 
@@ -1210,398 +1368,3 @@ _dbus_g_type_specialized_builtins_init (void)
   _dbus_g_type_register_map ("GHashTable", &hashtable_vtable, 0);
   _dbus_g_type_register_struct ("GValueArray", &valuearray_vtable, 0);
 }
-
-#ifdef DBUS_BUILD_TESTS
-
-typedef struct
-{
-  gboolean seen_foo;
-  gboolean seen_baz;
-} TestSpecializedHashData;
-
-static void
-test_specialized_hash (const GValue *key, const GValue *val, gpointer user_data)
-{
-  TestSpecializedHashData *data = user_data;
-
-  g_assert (G_VALUE_HOLDS_STRING (key));
-  g_assert (G_VALUE_HOLDS_STRING (val));
-
-  if (!strcmp (g_value_get_string (key), "foo"))
-    {
-      data->seen_foo = TRUE;
-      g_assert (!strcmp (g_value_get_string (val), "bar"));
-    }
-  else if (!strcmp (g_value_get_string (key), "baz"))
-    {
-      data->seen_baz = TRUE;
-      g_assert (!strcmp (g_value_get_string (val), "moo"));
-    }
-  else
-    {
-      g_assert_not_reached ();
-    }
-}
-
-static void
-test_specialized_hash_2 (const GValue *key, const GValue *val, gpointer user_data)
-{
-  TestSpecializedHashData *data = user_data;
-  const GValue *realval;
-
-  g_assert (G_VALUE_HOLDS_STRING (key));
-  g_assert (G_VALUE_TYPE (val) == G_TYPE_VALUE);
-
-  realval = g_value_get_boxed (val);
-
-  if (!strcmp (g_value_get_string (key), "foo"))
-    {
-      data->seen_foo = TRUE;
-      g_assert (G_VALUE_HOLDS_UINT (realval));
-      g_assert (g_value_get_uint (realval) == 20);
-    }
-  else if (!strcmp (g_value_get_string (key), "baz"))
-    {
-      data->seen_baz = TRUE;
-      g_assert (G_VALUE_HOLDS_STRING (realval));
-      g_assert (!strcmp ("bar", g_value_get_string (realval)));
-    }
-  else
-    {
-      g_assert_not_reached ();
-    }
-}
-
-gboolean
-_dbus_gvalue_utils_test (const char *datadir)
-{
-  GType type;
-
-  dbus_g_type_specialized_init ();
- _dbus_g_type_specialized_builtins_init ();
-
-  type = dbus_g_type_get_collection ("GArray", G_TYPE_UINT);
-  g_assert (dbus_g_type_is_collection (type));
-  g_assert (dbus_g_type_get_collection_specialization (type) == G_TYPE_UINT);
-  {
-    GArray *instance;
-
-    instance = dbus_g_type_specialized_construct (type);
-
-    g_assert (instance->len == 0);
-
-    g_array_free (instance, TRUE);
-  }
-
-  type = dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_STRING);
-  g_assert (dbus_g_type_is_map (type));
-  g_assert (dbus_g_type_get_map_key_specialization (type) == G_TYPE_STRING);
-  g_assert (dbus_g_type_get_map_value_specialization (type) == G_TYPE_STRING);
-  {
-    GHashTable *instance;
-    GValue val = { 0, };
-    TestSpecializedHashData hashdata;
-
-    instance = dbus_g_type_specialized_construct (type);
-
-    g_assert (g_hash_table_size (instance) == 0);
-    g_hash_table_insert (instance, g_strdup ("foo"), g_strdup ("bar"));
-    g_hash_table_insert (instance, g_strdup ("baz"), g_strdup ("moo"));
-    g_assert (g_hash_table_size (instance) == 2);
-
-    g_value_init (&val, type);
-    g_value_take_boxed (&val, instance);
-    hashdata.seen_foo = FALSE;
-    hashdata.seen_baz = FALSE;
-    dbus_g_type_map_value_iterate (&val,
-				   test_specialized_hash, 
-				   &hashdata);
-    
-    g_assert (hashdata.seen_foo);
-    g_assert (hashdata.seen_baz);
-
-    g_value_unset (&val);
-  }
-
-  type = dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE);
-  g_assert (dbus_g_type_is_map (type));
-  g_assert (dbus_g_type_get_map_key_specialization (type) == G_TYPE_STRING);
-  g_assert (dbus_g_type_get_map_value_specialization (type) == G_TYPE_VALUE);
-  {
-    GHashTable *instance;
-    GValue val = { 0, };
-    TestSpecializedHashData hashdata;
-    DBusGTypeSpecializedAppendContext ctx;
-    GValue *eltval;
-
-    instance = dbus_g_type_specialized_construct (type);
-    g_value_init (&val, type);
-    g_value_take_boxed (&val, instance);
-
-    dbus_g_type_specialized_init_append (&val, &ctx);
-
-    {
-      GValue keyval = { 0, };
-      GValue valval = { 0, };
-      g_value_init (&keyval, G_TYPE_STRING);
-      g_value_set_string (&keyval, "foo"); 
-
-      g_value_init (&valval, G_TYPE_VALUE);
-      eltval = g_new0 (GValue, 1);
-      g_value_init (eltval, G_TYPE_UINT);
-      g_value_set_uint (eltval, 20);
-      g_value_take_boxed (&valval, eltval);
-      dbus_g_type_specialized_map_append (&ctx, &keyval, &valval);
-    }
-
-    {
-      GValue keyval = { 0, };
-      GValue valval = { 0, };
-      g_value_init (&keyval, G_TYPE_STRING);
-      g_value_set_string (&keyval, "baz"); 
-      g_value_init (&valval, G_TYPE_VALUE);
-      eltval = g_new0 (GValue, 1);
-      g_value_init (eltval, G_TYPE_STRING);
-      g_value_set_string (eltval, "bar");
-      g_value_take_boxed (&valval, eltval);
-      dbus_g_type_specialized_map_append (&ctx, &keyval, &valval);
-    }
-
-    hashdata.seen_foo = FALSE;
-    hashdata.seen_baz = FALSE;
-    dbus_g_type_map_value_iterate (&val,
-				   test_specialized_hash_2, 
-				   &hashdata);
-    
-    g_assert (hashdata.seen_foo);
-    g_assert (hashdata.seen_baz);
-
-    g_value_unset (&val);
-  }
-
-  type = dbus_g_type_get_collection ("GSList", G_TYPE_OBJECT);
-  g_assert (dbus_g_type_is_collection (type));
-  g_assert (dbus_g_type_get_collection_specialization (type) == G_TYPE_OBJECT);
-  {
-    GSList *instance, *tmp, *copy;
-    GValue val = {0, };
-    GValue copyval = {0, };
-    DBusGTypeSpecializedAppendContext ctx;
-    GObject *objects[3];
-    int i;
-
-    instance = dbus_g_type_specialized_construct (type);
-    g_assert (instance == NULL);
-
-    g_value_init (&val, type);
-    g_value_take_boxed (&val, instance);
-
-    dbus_g_type_specialized_init_append (&val, &ctx);
-
-    for (i = 0; i < 3; i++)
-      {
-        GValue eltval = { 0, };
-        GObject *obj = g_object_new (G_TYPE_OBJECT, NULL);
-
-        g_assert (obj != NULL);
-        objects[i] = obj;
-        g_object_add_weak_pointer (obj, (gpointer) (objects + i));
-
-        g_value_init (&eltval, G_TYPE_OBJECT);
-        g_value_take_object (&eltval, obj);
-        dbus_g_type_specialized_collection_append (&ctx, &eltval);
-      }
-
-    dbus_g_type_specialized_collection_end_append (&ctx);
-
-    instance = g_value_get_boxed (&val);
-    g_assert (g_slist_length (instance) == 3);
-
-    for (tmp = instance; tmp; tmp = tmp->next)
-      {
-        GObject *obj = tmp->data;
-        g_assert (G_IS_OBJECT (obj));
-        g_assert (obj->ref_count == 1);
-      }
-
-    g_value_init (&copyval, type);
-    g_value_copy (&val, &copyval);
-
-    copy = g_value_get_boxed (&copyval);
-    g_assert (g_slist_length (copy) == 3);
-
-    for (tmp = copy; tmp; tmp = tmp->next)
-      {
-        GObject *obj = tmp->data;
-        g_assert (G_IS_OBJECT (obj));
-        g_assert (obj->ref_count == 2);
-      }
-
-    g_value_unset (&copyval);
-
-    for (i = 0; i < 3; i++)
-      {
-        g_assert (objects[i] != NULL);
-      }
-
-    for (tmp = instance; tmp; tmp = tmp->next)
-      {
-        GObject *obj = tmp->data;
-        g_assert (G_IS_OBJECT (obj));
-        g_assert (obj->ref_count == 1);
-      }
-
-    g_value_unset (&val);
-
-    for (i = 0; i < 3; i++)
-      {
-        g_assert (objects[i] == NULL);
-      }
-  }
-
-  type = dbus_g_type_get_collection ("GPtrArray", G_TYPE_STRING);
-  g_assert (dbus_g_type_is_collection (type));
-  g_assert (dbus_g_type_get_collection_specialization (type) == G_TYPE_STRING);
-  {
-    GPtrArray *instance;
-    DBusGTypeSpecializedAppendContext ctx;
-    GValue val = {0, };
-    GValue eltval = {0, };
-
-    instance = dbus_g_type_specialized_construct (type);
-
-    g_assert (instance->len == 0);
-
-    g_value_init (&val, type);
-    g_value_take_boxed (&val, instance);
-
-    dbus_g_type_specialized_init_append (&val, &ctx);
-
-    g_value_init (&eltval, G_TYPE_STRING);
-    g_value_set_static_string (&eltval, "foo");
-    dbus_g_type_specialized_collection_append (&ctx, &eltval);
-
-    g_value_reset (&eltval);
-    g_value_set_static_string (&eltval, "bar");
-    dbus_g_type_specialized_collection_append (&ctx, &eltval);
-
-    g_value_reset (&eltval);
-    g_value_set_static_string (&eltval, "baz");
-    dbus_g_type_specialized_collection_append (&ctx, &eltval);
-
-    dbus_g_type_specialized_collection_end_append (&ctx);
-
-    g_assert (instance->len == 3);
-
-    g_assert (!strcmp ("foo", g_ptr_array_index (instance, 0)));
-    g_assert (!strcmp ("bar", g_ptr_array_index (instance, 1)));
-    g_assert (!strcmp ("baz", g_ptr_array_index (instance, 2)));
-
-    g_value_unset (&val);
-  }
-
-  type = dbus_g_type_get_struct ("GValueArray", G_TYPE_STRING, G_TYPE_UINT, DBUS_TYPE_G_OBJECT_PATH, G_TYPE_INVALID);
-  g_assert (dbus_g_type_is_struct (type));
-  g_assert (dbus_g_type_get_struct_size (type) == 3);
-  g_assert (dbus_g_type_get_struct_member_type (type, 0) == G_TYPE_STRING);
-  g_assert (dbus_g_type_get_struct_member_type (type, 1) == G_TYPE_UINT);
-  g_assert (dbus_g_type_get_struct_member_type (type, 2) == DBUS_TYPE_G_OBJECT_PATH);
-  {
-    GValueArray *instance;
-    GValue val = {0, };
-    GValue memval = {0, };
-
-    instance = dbus_g_type_specialized_construct (type);
-
-    g_assert (instance->n_values == 3);
-
-    g_value_init (&val, type);
-    g_value_take_boxed (&val, instance);
-
-    g_value_init (&memval, G_TYPE_STRING);
-    g_value_set_static_string (&memval, "foo");
-    dbus_g_type_struct_set_member (&val, 0, &memval);
-    g_value_unset (&memval);
-
-    g_value_init (&memval, G_TYPE_UINT);
-    g_value_set_uint (&memval, 42);
-    dbus_g_type_struct_set_member (&val, 1, &memval);
-    g_value_unset (&memval);
-
-    g_value_init (&memval, DBUS_TYPE_G_OBJECT_PATH);
-    g_value_set_static_boxed (&memval, "/bar/moo/foo/baz");
-    dbus_g_type_struct_set_member (&val, 2, &memval);
-    g_value_unset (&memval);
-
-    g_assert (instance->n_values == 3);
-
-    g_value_init (&memval, G_TYPE_STRING);
-    dbus_g_type_struct_get_member (&val, 0, &memval);
-    g_assert (0 == strcmp (g_value_get_string (&memval), "foo"));
-    g_value_unset (&memval);
-
-    g_value_init (&memval, G_TYPE_UINT);
-    dbus_g_type_struct_get_member (&val, 1, &memval);
-    g_assert (g_value_get_uint (&memval) == 42);
-    g_value_unset (&memval);
-
-    g_value_init (&memval, DBUS_TYPE_G_OBJECT_PATH);
-    dbus_g_type_struct_get_member (&val, 2, &memval);
-    g_assert (0 == strcmp ((gchar*) g_value_get_boxed (&memval),
-                           "/bar/moo/foo/baz"));
-    g_value_unset (&memval);
-
-    g_value_unset (&val);
-  }
-
-  type = dbus_g_type_get_struct ("GValueArray", G_TYPE_STRING, G_TYPE_UINT, DBUS_TYPE_G_OBJECT_PATH, G_TYPE_INVALID);
-  g_assert (dbus_g_type_is_struct (type));
-  g_assert (dbus_g_type_get_struct_size (type) == 3);
-  g_assert (dbus_g_type_get_struct_member_type (type, 0) == G_TYPE_STRING);
-  g_assert (dbus_g_type_get_struct_member_type (type, 1) == G_TYPE_UINT);
-  g_assert (dbus_g_type_get_struct_member_type (type, 2) == DBUS_TYPE_G_OBJECT_PATH);
-  {
-    GValueArray *instance;
-    GValue val = {0, };
-
-    instance = dbus_g_type_specialized_construct (type);
-
-    g_assert (instance->n_values == 3);
-
-    g_value_init (&val, type);
-    g_value_take_boxed (&val, instance);
-
-    dbus_g_type_struct_set (&val,
-                            0,"foo",
-                            1, 42,
-                            2, "/bar/moo/foo/baz",
-                            G_MAXUINT);
-
-    g_assert (instance->n_values == 3);
-
-    {
-      gchar *string;
-      guint intval;
-      gchar *path;
-
-      dbus_g_type_struct_get (&val,
-                              0, &string,
-                              1, &intval,
-                              2, &path,
-                              G_MAXUINT);
-
-      g_assert (0 == strcmp (string, "foo"));
-      g_assert (intval == 42);
-      g_assert (0 == strcmp (path, "/bar/moo/foo/baz"));
-    }
-
-    g_value_unset (&val);
-  }
-
-
-  return TRUE;
-}
-
-
-
-#endif /* DBUS_BUILD_TESTS */
