@@ -42,7 +42,7 @@
 
 #include "my-object.h"
 
-#include "test/lib/util.h"
+#include "dbus-gmain/tests/util.h"
 
 GMainLoop *loop = NULL;
 
@@ -91,13 +91,13 @@ teardown (Fixture *f,
    * in test_lookup() */
   if (f->bus != NULL)
     {
-      test_run_until_disconnected (f->bus, NULL);
+      test_run_until_disconnected (dbus_g_connection_get_connection (f->bus), NULL);
       dbus_g_connection_unref (f->bus);
     }
 
   if (f->bus2 != NULL)
     {
-      test_run_until_disconnected (f->bus2, NULL);
+      test_run_until_disconnected (dbus_g_connection_get_connection (f->bus2), NULL);
       dbus_g_connection_unref (f->bus2);
     }
 
